package ab.demo;

/*****************************************************************************
** ANGRYBIRDS AI AGENT FRAMEWORK
** Copyright (c) 2013, Sahan Abeyasinghe, XiaoYu (Gary) Ge, Stephen Gould,
** Jim Keys, Kar-Wai Lim, Zain Mubashir, Jochen Renz, Andrew Wang, Peng Zhang
** All rights reserved.
**This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License. 
**To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-sa/3.0/ 
*or send a letter to Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
*****************************************************************************/

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.UnknownHostException;
import java.util.List;

import javax.imageio.ImageIO;

import ab.server.Proxy;
import ab.server.proxy.message.ProxyScreenshotMessage;
import ab.utils.ShowDebuggingImage;
import ab.vision.GameStateExtractor;
import ab.vision.Vision;
import ab.vision.VisionUtils;

public class abAgent {
    private static Proxy server;

    public abAgent() {
        if (server == null) {
            try {
		server = new Proxy(9000) {
			@Override
                        public void onOpen() {
                            System.out.println("Client connected");
			}

			@Override
			public void onClose() {
                            System.out.println("Client disconnected");
			}
                    };
		server.start();

		System.out.println("Server started on port: " + server.getPort());

		System.out.println("Waiting for client to connect");
		server.waitForClients(1);

            } catch (UnknownHostException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
            }
        }
    }

    public BufferedImage doScreenShot() {
	byte[] imageBytes = server.send(new ProxyScreenshotMessage());
        BufferedImage image = null;
        try {
            image = ImageIO.read(new ByteArrayInputStream(imageBytes));
        } catch (IOException e) {
            // do something
        }

        return image;
    }

    public static void main(String args[]) {
        abAgent ar = new abAgent();

        ShowDebuggingImage frame = null;
        GameStateExtractor gameStateExtractor = new GameStateExtractor();

        while (true) {
            // capture image
            BufferedImage screenshot = ar.doScreenShot();
            final int nHeight = screenshot.getHeight();
            final int nWidth = screenshot.getWidth();

            System.out.println("captured image of size " + nWidth + "-by-" + nHeight);

            // meta debugging information
            int[][] meta = new int[nHeight][nWidth];
            for (int y = 0; y < nHeight; y++) {
                for (int x = 0; x < nWidth; x++) {
                    final int colour = screenshot.getRGB(x, y);
                    meta[y][x] = ((colour & 0x00e00000) >> 15) | ((colour & 0x0000e000) >> 10) | ((colour & 0x000000e0) >> 5);
                }
            }

            // extract game state
            GameStateExtractor.GameState state = gameStateExtractor.getGameState(screenshot);
            System.out.println(state);
            if (state == GameStateExtractor.GameState.MAIN_MENU) {
                // do nothing
            } else if (state == GameStateExtractor.GameState.EPISODE_MENU) {
                // do nothing
            } else if (state == GameStateExtractor.GameState.LEVEL_SELECTION) {
                // do nothing
            } else if (state == GameStateExtractor.GameState.LOADING) {
                // do nothing
            } else if (state == GameStateExtractor.GameState.WON) {
                // extract score
                System.out.println("...won with score " + gameStateExtractor.getScoreEndGame(screenshot));
            } else if (state == GameStateExtractor.GameState.LOST) {
                System.out.println("...lost");
            } else if (state == GameStateExtractor.GameState.PLAYING) {
                // extract score
                System.out.println("...current score is " + gameStateExtractor.getScoreInGame(screenshot));

                // process image
                Vision vision = new Vision(screenshot);
                List<Rectangle> pigs = vision.findPigs();
                List<Rectangle> redBirds = vision.findRedBirds();
                List<Rectangle> blueBirds = vision.findBlueBirds();
                List<Rectangle> yellowBirds = vision.findYellowBirds();
                
                Rectangle sling = vision.findSlingshot();
                System.out.println("...found " + pigs.size() + " pigs and " + 
                    (redBirds.size() + blueBirds.size() + yellowBirds.size()) + " birds");

                // convert screenshot to greyscale and draw bounding boxes
                screenshot = VisionUtils.convert2grey(screenshot);
                VisionUtils.drawBoundingBoxes(screenshot, pigs, Color.GREEN);
                VisionUtils.drawBoundingBoxes(screenshot, redBirds, Color.RED);
                VisionUtils.drawBoundingBoxes(screenshot, blueBirds, Color.BLUE);
                VisionUtils.drawBoundingBoxes(screenshot, yellowBirds, Color.YELLOW);
                if (sling != null) {
                    VisionUtils.drawBoundingBox(screenshot, sling, Color.ORANGE);
                }
            }

            if (state != GameStateExtractor.GameState.PLAYING) {
                screenshot = VisionUtils.convert2grey(screenshot);
            }

            // show image
            if (frame == null) {
                frame = new ShowDebuggingImage("abAgent", screenshot, meta);
            } else {
                frame.refresh(screenshot, meta);
            }

            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
            }
        }

        //System.exit(0);
    }
}
